##
# Get a specific variable for a package, falling back to the global one if no
# package-specific version exists
#
# Arguments:
#   variable:   Variable to find
#   package:    Package to look up variable for
#   info:       Dict to look up in
#
def get_variable(variable, package, info):
    if package and package in info['packages']:
        package = info['packages'][package]
        if variable in package:
            return package[variable]

    if variable in info:
        return info[variable]

    else:
        return None


##
# Return the info for a single package
#
def get_merged_package(package, srcinfo):
    if not package in srcinfo['packages']:
        return None

    merged = srcinfo.copy()
    package_info = merged['packages'][package]
    for key in package_info:
        merged[key] = package_info[key]

    merged.pop('packages')
    merged['pkgname'] = package

    return merged


##
# Get a list of package names
#
def get_package_names(info):
    return [p for p in info['packages'].keys()]



from srcinfo.parse import parse_srcinfo
import sys


##
# Parse SRCINFO from stdin and pretty-print parsed info
#
def main():
    output_json = len(sys.argv) > 1 and sys.argv[1] == "--json"

    srcinfo = sys.stdin.read()
    (info, errors) = parse_srcinfo(srcinfo)

    if output_json:
        if errors:
            sys.exit(1)
        else:
            import json
            print(json.dumps(info))
    else:
        import pprint
        pprint.PrettyPrinter().pprint(info)
        if errors:
            print('==> The following errors occured while parsing:', file=sys.stderr)
            for error in errors:
                print(error, file=sys.stderr)



from parse import parse

arrays = ['pkgname', 'arch', 'license', 'groups', 'options',
          'conflicts', 'provides', 'replaces',
          'source', 'noextract', 'backup', 'validpgpkeys', 'b2sums', 'md5sums',
          'sha1sums', 'sha224sums', 'sha256sums', 'sha384sums', 'sha512sums',
          'depends', 'makedepends', 'checkdepends', 'optdepends',
          ]


##
# Check if SRCINFO key is an array
#
def is_array(key):
    if key in arrays:
        return True
    for array in arrays:
        if key.startswith(array):
            return True
    return False


##
# Insert `value` into the list index `key` in list `target`
#
def list_insert(target, key, value):
    if key not in target:
        target[key] = []

    # Don't append value if already in the list
    if value and value not in target[key]:
        target[key].append(value)


##
# Extract a SRCINFO variable from `string`
#
def extract_var(string):
    line = string.lstrip()
    if line.startswith('#'):
        return (None, None, None)

    key = None
    value = None
    errors = []

    def value(string):
        return string.strip()
    value.pattern = r'.*'

    parsed = parse('{:^}={:value}', line, dict(value=value))
    if parsed:
        (key, value) = parsed
    else:
        errors.append('failed to parse line: {!r}'.format(line))

    return (key, value, errors)


##
# Parse SRCINFO from string
#
# Returns a tuple of the srcinfo dict and an list of any errors found while
# parsing
#
def parse_srcinfo(source):
    lines = source.splitlines()

    srcinfo = {'packages': {}}
    info = srcinfo
    errors = []
    for index, line in enumerate(lines, start=1):
        if not line.strip():
            continue

        (key, value, err) = extract_var(line)

        if err:
            errors.append({
                'line': index,
                'error': err
            })

        if key == 'pkgbase':
            if 'pkgbase' in srcinfo:
                errors.append({
                    'line': index,
                    'error': "pkgbase declared more than once",
                })

            elif info != srcinfo:
                errors.append({
                    'line': index,
                    'error': "pkgbase declared after pkgname",
                })

        if key == 'pkgname':
            pkgname = value
            info = srcinfo['packages'][pkgname] = {}
            continue

        elif not key:
            continue

        if is_array(key):
            list_insert(info, key, value)
        else:
            info[key] = value

    return (srcinfo, errors)
